Package: doFuture
Version: 1.2.2
Title: Use Foreach to Parallelize via the Future Framework
Depends: foreach (>= 1.5.0), future (>= 1.49.0)
Imports: future.apply, globals, iterators, parallel, utils
Suggests: doRNG (>= 1.8.2), markdown, R.rsp
VignetteBuilder: R.rsp
Authors@R: c(person("Henrik", "Bengtsson",
                    role = c("aut", "cre", "cph"),
                    email = "henrikb@braju.com",
                    comment = c(ORCID = "0000-0002-7579-5165")))
Description: The 'future' package provides a unifying parallelization framework for R that supports many parallel and distributed backends <doi:10.32614/RJ-2021-048>. The 'foreach' package provides a powerful API for iterating over an R expression in parallel. The 'doFuture' package brings the best of the two together. There are two alternative ways to use this package. The recommended approach is to use 'y <- foreach(...) %dofuture% { ... }', which does not require using 'registerDoFuture()' and has many advantages over '%dopar%'. The alternative is the traditional 'foreach' approach by registering the 'foreach' adapter 'registerDoFuture()' and so that 'y <- foreach(...) %dopar% { ... }' runs in parallelizes with the 'future' framework.
License: LGPL (>= 2.1)
LazyLoad: TRUE
URL: https://doFuture.futureverse.org,
        https://github.com/futureverse/doFuture
BugReports: https://github.com/futureverse/doFuture/issues
Language: en-US
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-22 06:29:01 UTC; hb
Author: Henrik Bengtsson [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-7579-5165>)
Maintainer: Henrik Bengtsson <henrikb@braju.com>
Repository: CRAN
Date/Publication: 2026-05-22 07:40:02 UTC
Built: R 4.5.2; ; 2026-05-22 11:22:08 UTC; unix
