# Generated by roxygen2: do not edit by hand

S3method(with,DoPar)
export("%dofuture%")
export(registerDoFuture)
export(withDoRNG)
import(future.apply)
importFrom(foreach,getErrorIndex)
importFrom(foreach,getErrorValue)
importFrom(foreach,getResult)
importFrom(foreach,makeAccum)
importFrom(foreach,registerDoSEQ)
importFrom(foreach,setDoPar)
importFrom(future,Future)
importFrom(future,FutureError)
importFrom(future,FutureInterruptError)
importFrom(future,cancel)
importFrom(future,future)
importFrom(future,getGlobalsAndPackages)
importFrom(future,nbrOfWorkers)
importFrom(future,resolve)
importFrom(future,value)
importFrom(globals,globalsByName)
importFrom(iterators,iter)
importFrom(parallel,splitIndices)
importFrom(utils,capture.output)
importFrom(utils,globalVariables)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,str)
