#! /usr/bin/env Rscript
## This runs testme test script inst/testme/test-foreach_dofuture,nested_colon.R
## Don't edit - it was autogenerated by inst/testme/deploy.R
doFuture:::testme("foreach_dofuture,nested_colon")
